unit TrekTag;

{**********************************************************}
{                                                          }
{  TTrekTag Unit                                           }
{  Copyright 1999 Workshell Software.                     }
{							   }
{  Version 1.0						   }
{                                                          }
{                                                          }
{  Web      -> http://www.workshell.uni.cc/                }
{  E - Mail -> trekvcl@kinsella.u-net.com                  }
{                                                          }
{**********************************************************}

interface

{$I TrekVer.inc}

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

type
 TTagLayout = (tlLeft, tlRight, tlUp, tlDown);

type
  TTrekTag = class(TCustomControl)
  private
    { Private declarations }
    FLayout: TTagLayout;
    FDivider: Boolean;
    FOnMouseEnter: TNotifyEvent;
    FOnMouseLeave: TNotifyEvent;
    FColor: TColor;
    FBorderColor: TColor;
    FBackgroundColor: TColor;
    procedure SetLayout(Value: TTagLayout);
    procedure SetDivider(Value: Boolean);
    procedure SetColor(Value: TColor);
    procedure SetBorderColor(Value: TColor);
    procedure SetBackgroundColor(Value: TColor);
    procedure SetCaption(var Message: TMessage); message CM_TEXTCHANGED;
    procedure SetFont(var Message: TMessage);     message CM_FONTCHANGED;
    procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
    procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
  protected
    { Protected declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Paint; override;
  public
    { Public declarations }
  published
    { Published declarations }
    property Divider: Boolean read FDivider write SetDivider;
    property Layout: TTagLayout read FLayout write SetLayout;
    property Color: TColor read FColor write SetColor;
    property BorderColor: TColor read FBorderColor write SetBorderColor;
    property BackgroundColor: TColor read FBackgroundColor write SetBackgroundColor;
    property OnMouseEnter: TNotifyEvent read FOnMouseEnter write FOnMouseEnter;
    property OnMouseLeave: TNotifyEvent read FOnMouseLeave write FOnMouseLeave;
    property Caption;
    property Align;
    property Enabled;
    property Font;
    property OnClick;
    property OnDblClick;
    property OnMouseDown;
    property OnMouseUp;
    property OnMouseMove;
   {$IFDEF D4CB4}
    property Anchors;
    property BiDiMode;
    property Constraints;
    property DragKind;
    property ParentBiDiMode;
    property OnEndDock;
    property OnStartDock;
   {$ENDIF}
  end;

procedure Register;

implementation

constructor TTrekTag.Create(AOwner: TComponent);
begin
 inherited Create(AOwner);
  Width := 25;
  Height := 25;
  Color := clBlack;
  BorderColor := $00CF9030;
  BackgroundColor := clBlack;
  Layout := tlLeft;
  ControlStyle := [csAcceptsControls, csCaptureMouse, csClickEvents,
  csOpaque, csDoubleClicks, csReplicatable];
end;

destructor TTrekTag.Destroy;
begin
 inherited Destroy;
end;

procedure TTrekTag.Paint;
var
 Bitmap: TBitmap;
 X, Y, W: Integer;

procedure DrawCaption(Placement:integer);
var
 NewCaption:string;
 X, Y, xLoc, yLoc: Integer;
begin
NewCaption := Caption;
X := Width div 2;
Y := Height div 2;
xLoc := 0;
yLoc := 0;
 with inherited Canvas do
  begin
   if Placement = 1 then
    begin
     xLoc := X - (TextWidth(NewCaption) div 2);
     yLoc := Y - (TextHeight(NewCaption) div 2);
    end;
   if Placement = 2 then
    begin
     xLoc := X - (TextWidth(NewCaption) div 2);
     yLoc := Y - (TextHeight(NewCaption) div 2);
    end;
   if Placement = 3 then
    begin
     xLoc := X - (TextWidth(NewCaption)div 2);
     yLoc := Y - (TextHeight(NewCaption) div 2);
    end;
   if Placement = 4 then
    begin
     xLoc := X - (TextWidth(NewCaption)div 2);
     yLoc := Y - (TextHeight(NewCaption) div 2);
    end;
   TextOut(xLoc,yLoc,NewCaption);
  end;
end;

begin
Bitmap := TBitmap.Create;
Bitmap.Width := Width;
Bitmap.Height := Height;
with Bitmap.Canvas do
 begin
  Brush.Color := FBackgroundColor;
  FillRect(Rect(0,0,Width,Height));
  Canvas.Font.Assign(Self.Font);
  Pen.Color := FBorderColor;
  Brush.Color := FColor;
  X := Width div 2;
  Y := Height div 2;
  W := Pen.Width;

  if FLayout = tlLeft then
   begin
    if Height > Width then
     begin
      Chord(W, W, Width - W, Height - W, X, W, X, Height - W);
      Rectangle(X, W, Width - W, Height - W);
      if not FDivider then
       begin
        Pen.Color := Brush.Color;
        MoveTo(X, W + 1);
        LineTo(X, Width - W - 1);
       end;
     end
   else
 begin
  Chord(W, W, Height - W, Height - W, Y, W, Y, Height - W);
  Rectangle(Y, W, Width - W, Height - W);
  if not FDivider then
   begin
    Pen.Color := Brush.Color;
    MoveTo(Y, W + 1);
    LineTo(Y, Height - W - 1);
   end;
 end;
 DrawCaption(1);
end;

if FLayout = tlRight then
 begin
   if Height > Width then
    begin
     Chord(W, W, Width - W, Height - W, X, Height - W, X, W);
     Rectangle(W, W, X + 1, Height - W);
    if not FDivider then
     begin
      Pen.Color := Brush.Color;
      MoveTo(X, W + 1);
      LineTo(X, Height - W - 1);
     end;
    end
   else
 begin
  Chord(Width - Height, W, Width - W, Height - W, Width - Y, Height - W, Width - Y, W);
  Rectangle(W, W, Width - Y + 1, Height - W);
  if not FDivider then
   begin
    Pen.Color := Brush.Color;
    MoveTo(Width - Y, W + 1);
    LineTo(Width - Y, Height - W - 1);
   end;
 end;
DrawCaption(2);
end;

if FLayout = tlUp then
 begin
  if Width > Height then
   begin
    Chord(W, W, Width - W, Height - W, Width - W, Y, W, Y);
    Rectangle(W, Y, Width - W, Height - W);
    if not FDivider then
     begin
      Pen.Color := Brush.Color;
      MoveTo(W + 1, Y);
      LineTo(Width - W - 1, Y);
     end;
    end
   else
 begin
  Chord(W, W, Width - W, Width - W, Width - W, X, W, X);
  Rectangle(W, X, Width - W, Height - W);
  if not FDivider then
   begin
    Pen.Color := Brush.Color;
    MoveTo(W + 1, X);
    LineTo(Width - W - 1, X);
   end;
 end;
DrawCaption(3);
end;

if FLayout = tlDown then
 begin
  if  Width > Height then
   begin
    Chord(W, W, Width - W, Height - W, W, Y, Width - W, Y);
    Rectangle(W, W, Width - W, Y + 1);
    if not FDivider then
     begin
      Pen.Color := Brush.Color;
      MoveTo(W + 1, Y);
      LineTo(Width - W - 1, Y);
     end;
    end
   else
 begin
  Chord(W, Height - Width, Width - W, Height - W, W, Height - X, Width - W, Height - X);
  Rectangle(W, W, Width - W, Height - X + 1);
  if not FDivider then
   begin
    Pen.Color := Brush.Color;
    MoveTo(W + 1, Height - X);
    LineTo(Width - W - 1, Height - X);
   end;
 end;
DrawCaption(4);
end;
end;
Canvas.CopyMode := cmSrcCopy;
Canvas.Draw(0,0, Bitmap);
Bitmap.Free;
end;

procedure TTrekTag.SetDivider(Value: Boolean);
begin
 FDivider := Value;
 Repaint;
end;

procedure TTrekTag.SetLayout(Value: TTagLayout);
begin
 FLayout := Value;
 Repaint;
end;

procedure TTrekTag.SetColor(Value: TColor);
begin
 FColor := Value;
 Repaint;
end;

procedure TTrekTag.SetBorderColor(Value: TColor);
begin
 FBorderColor := Value;
 Repaint;
end;

procedure TTrekTag.SetBackgroundColor(Value: TColor);
begin
 FBackgroundColor := Value;
 Repaint;
end;

procedure TTrekTag.SetCaption(var Message: TMessage);
begin
 Repaint;
end;

procedure TTrekTag.SetFont(var Message: TMessage);
begin
 inherited;
 Repaint;
end;

procedure TTrekTag.CMMouseEnter(var Message: TMessage);
begin
if not (csDesigning in ComponentState) and (Enabled = True) then
 begin
  if Assigned(FOnMouseEnter) then FOnMouseEnter(Self);
 end;
end;

procedure TTrekTag.CMMouseLeave(var Message: TMessage);
begin
if not (csDesigning in ComponentState) and (Enabled = True) then
 begin
  if Assigned(FOnMouseLeave) then FOnMouseLeave(Self);
 end;
end;

procedure Register;
begin
 RegisterComponents('StarTrek',[TTrekTag]);
end;

end.
